"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrapLines = exports.renderText = void 0;
var text_utils_1 = require("../../../../../common/text_utils");
var canvas_1 = require("../../../../../renderers/canvas");
function renderText(ctx, origin, text, font, degree, translation, scale) {
    if (degree === void 0) { degree = 0; }
    if (scale === void 0) { scale = 1; }
    canvas_1.withRotatedOrigin(ctx, origin, degree, function (ctx) {
        canvas_1.withContext(ctx, function (ctx) {
            var _a, _b, _c;
            ctx.fillStyle = font.fill;
            ctx.textAlign = font.align;
            ctx.textBaseline = font.baseline;
            ctx.font = text_utils_1.cssFontShorthand(font, font.fontSize);
            if ((translation === null || translation === void 0 ? void 0 : translation.x) || (translation === null || translation === void 0 ? void 0 : translation.y)) {
                ctx.translate((_a = translation === null || translation === void 0 ? void 0 : translation.x) !== null && _a !== void 0 ? _a : 0, (_b = translation === null || translation === void 0 ? void 0 : translation.y) !== null && _b !== void 0 ? _b : 0);
            }
            ctx.translate(origin.x, origin.y);
            ctx.scale(scale, scale);
            var shadowSize = (_c = font.shadowSize) !== null && _c !== void 0 ? _c : 0;
            if (font.shadow && shadowSize > 0) {
                ctx.lineJoin = 'round';
                ctx.lineWidth = shadowSize;
                ctx.strokeStyle = font.shadow;
                ctx.strokeText(text, 0, 0);
            }
            ctx.fillText(text, 0, 0);
        });
    });
}
exports.renderText = renderText;
var SPACE = ' ';
var ELLIPSIS = '…';
var DASH = '-';
function wrapLines(ctx, text, font, fontSize, fixedWidth, fixedHeight, _a) {
    var _b = _a === void 0 ? { wrapAtWord: true, shouldAddEllipsis: false } : _a, wrapAtWord = _b.wrapAtWord, shouldAddEllipsis = _b.shouldAddEllipsis;
    var lineHeight = 1;
    var lines = text.split('\n');
    var textWidth = 0;
    var lineHeightPx = lineHeight * fontSize;
    var padding = 0;
    var maxWidth = fixedWidth - padding * 2;
    var maxHeightPx = fixedHeight - padding * 2;
    var currentHeightPx = 0;
    var shouldWrap = true;
    var textArr = [];
    var textMeasureProcessor = text_utils_1.measureText(ctx);
    var getTextWidth = function (textString) {
        var measuredText = textMeasureProcessor(fontSize, [
            __assign({ text: textString }, font),
        ]);
        var _a = __read(measuredText, 1), measure = _a[0];
        if (measure) {
            return measure.width;
        }
        return 0;
    };
    var additionalWidth = shouldAddEllipsis ? getTextWidth(ELLIPSIS) : 0;
    for (var i = 0, max = lines.length; i < max; ++i) {
        var line = lines[i];
        var lineWidth = getTextWidth(line);
        if (fixedWidth && lineWidth > maxWidth) {
            while (line.length > 0) {
                var low = 0;
                var high = line.length;
                var match = '';
                var matchWidth = 0;
                while (low < high) {
                    var mid = (low + high) >>> 1;
                    var substr = line.slice(0, mid + 1);
                    var substrWidth = getTextWidth(substr) + additionalWidth;
                    if (substrWidth <= maxWidth) {
                        low = mid + 1;
                        match = substr + (shouldAddEllipsis ? ELLIPSIS : '');
                        matchWidth = substrWidth;
                    }
                    else {
                        high = mid;
                    }
                }
                if (match) {
                    if (wrapAtWord) {
                        var nextChar = line[match.length];
                        var nextIsSpaceOrDash = nextChar === SPACE || nextChar === DASH;
                        var wrapIndex = nextIsSpaceOrDash && matchWidth <= maxWidth
                            ? match.length
                            : Math.max(match.lastIndexOf(SPACE), match.lastIndexOf(DASH)) + 1;
                        if (wrapIndex > 0) {
                            low = wrapIndex;
                            match = match.slice(0, low);
                            matchWidth = getTextWidth(match);
                        }
                    }
                    match = match.trimEnd();
                    textArr.push(match);
                    textWidth = Math.max(textWidth, matchWidth);
                    currentHeightPx += lineHeightPx;
                    if (!shouldWrap || (fixedHeight && currentHeightPx + lineHeightPx > maxHeightPx)) {
                        break;
                    }
                    line = line.slice(low);
                    line = line.trimStart();
                    if (line.length > 0) {
                        lineWidth = getTextWidth(line);
                        if (lineWidth <= maxWidth) {
                            textArr.push(line);
                            currentHeightPx += lineHeightPx;
                            textWidth = Math.max(textWidth, lineWidth);
                            break;
                        }
                    }
                }
                else {
                    break;
                }
            }
        }
        else {
            textArr.push(line);
            currentHeightPx += lineHeightPx;
            textWidth = Math.max(textWidth, lineWidth);
        }
        if (fixedHeight && currentHeightPx + lineHeightPx > maxHeightPx) {
            break;
        }
    }
    return {
        lines: textArr,
        height: fontSize,
        width: textWidth,
    };
}
exports.wrapLines = wrapLines;
//# sourceMappingURL=text.js.map